@@ -53,7 +53,7 @@ def main(args: list[str]=sys.argv) -> None:
     patch_color_list('tools/cmd/at/set_colors.go', nullable_colors, 'NULLABLE')
     patch_color_list('tools/themes/collection.go', all_colors, 'ALL')
     nc = '\n    '.join(f'{x!r}' for x in nullable_colors)
-    write_output('kitty', definition, f'\nnullable_colors = frozenset({{\n    {nc}\n}})')
+    write_output('kitty', definition, f'\nnullable_colors = frozenset({{\n    {nc}\n}})\n')
 
 
 if __name__ == '__main__':
