@@ -4,9 +4,10 @@
 import os
 import sys
 from collections import defaultdict
+from collections.abc import Generator, Sequence
 from contextlib import contextmanager, suppress
 from itertools import count
-from typing import TYPE_CHECKING, DefaultDict, Dict, Generator, List, Optional, Sequence, Tuple
+from typing import TYPE_CHECKING, DefaultDict, Optional, TypedDict
 
 import kitty.fast_data_types as fast_data_types
 
@@ -14,10 +15,6 @@ from .constants import handled_signals, is_freebsd, is_macos, kitten_exe, kitty_
 from .types import run_once
 from .utils import cmdline_for_hold, log_error, which
 
-try:
-    from typing import TypedDict
-except ImportError:
-    TypedDict = dict
 if TYPE_CHECKING:
     from .window import CwdRequest
 
@@ -29,19 +26,21 @@ if is_macos:
     from kitty.fast_data_types import process_group_map as _process_group_map
 
     def cwd_of_process(pid: int) -> str:
+        # The underlying code on macos returns a path with symlinks resolved
+        # anyway but we use realpath for extra safety.
         return os.path.realpath(_cwd(pid))
 
-    def process_group_map() -> DefaultDict[int, List[int]]:
-        ans: DefaultDict[int, List[int]] = defaultdict(list)
+    def process_group_map() -> DefaultDict[int, list[int]]:
+        ans: DefaultDict[int, list[int]] = defaultdict(list)
         for pid, pgid in _process_group_map():
             ans[pgid].append(pid)
         return ans
 
-    def cmdline_of_pid(pid: int) -> List[str]:
+    def cmdline_of_pid(pid: int) -> list[str]:
         return cmdline_(pid)
 else:
 
-    def cmdline_of_pid(pid: int) -> List[str]:
+    def cmdline_of_pid(pid: int) -> list[str]:
         with open(f'/proc/{pid}/cmdline', 'rb') as f:
             return list(filter(None, f.read().decode('utf-8').split('\0')))
 
@@ -55,6 +54,8 @@ else:
             return os.path.realpath(ans)
     else:
         def cwd_of_process(pid: int) -> str:
+            # We use realpath instead of readlink to match macOS behavior where
+            # the underlying OS API returns real paths.
             ans = f'/proc/{pid}/cwd'
             return os.path.realpath(ans)
 
@@ -62,8 +63,8 @@ else:
         with open(f'/proc/{pid}/environ', 'rb') as f:
             return f.read().decode('utf-8')
 
-    def process_group_map() -> DefaultDict[int, List[int]]:
-        ans: DefaultDict[int, List[int]] = defaultdict(list)
+    def process_group_map() -> DefaultDict[int, list[int]]:
+        ans: DefaultDict[int, list[int]] = defaultdict(list)
         for x in os.listdir('/proc'):
             try:
                 pid = int(x)
@@ -87,8 +88,8 @@ def checked_terminfo_dir() -> Optional[str]:
     return terminfo_dir if os.path.isdir(terminfo_dir) else None
 
 
-def processes_in_group(grp: int) -> List[int]:
-    gmap: Optional[DefaultDict[int, List[int]]] = getattr(process_group_map, 'cached_map', None)
+def processes_in_group(grp: int) -> list[int]:
+    gmap: Optional[DefaultDict[int, list[int]]] = getattr(process_group_map, 'cached_map', None)
     if gmap is None:
         try:
             gmap = process_group_map()
@@ -110,11 +111,11 @@ def cached_process_data() -> Generator[None, None, None]:
         delattr(process_group_map, 'cached_map')
 
 
-def parse_environ_block(data: str) -> Dict[str, str]:
+def parse_environ_block(data: str) -> dict[str, str]:
     """Parse a C environ block of environment variables into a dictionary."""
     # The block is usually raw data from the target process.  It might contain
     # trailing garbage and lines that do not look like assignments.
-    ret: Dict[str, str] = {}
+    ret: dict[str, str] = {}
     pos = 0
 
     while True:
@@ -133,11 +134,11 @@ def parse_environ_block(data: str) -> Dict[str, str]:
     return ret
 
 
-def environ_of_process(pid: int) -> Dict[str, str]:
+def environ_of_process(pid: int) -> dict[str, str]:
     return parse_environ_block(_environ_of_process(pid))
 
 
-def process_env() -> Dict[str, str]:
+def process_env() -> dict[str, str]:
     ans = dict(os.environ)
     ssl_env_var = getattr(sys, 'kitty_ssl_env_var', None)
     if ssl_env_var is not None:
@@ -146,14 +147,14 @@ def process_env() -> Dict[str, str]:
     return ans
 
 
-def default_env() -> Dict[str, str]:
-    ans: Optional[Dict[str, str]] = getattr(default_env, 'env', None)
+def default_env() -> dict[str, str]:
+    ans: Optional[dict[str, str]] = getattr(default_env, 'env', None)
     if ans is None:
         return process_env()
     return ans
 
 
-def set_default_env(val: Optional[Dict[str, str]] = None) -> None:
+def set_default_env(val: Optional[dict[str, str]] = None) -> None:
     env = process_env().copy()
     has_lctype = False
     if val:
@@ -167,7 +168,7 @@ def set_LANG_in_default_env(val: str) -> None:
     default_env().setdefault('LANG', val)
 
 
-def openpty() -> Tuple[int, int]:
+def openpty() -> tuple[int, int]:
     master, slave = os.openpty()  # Note that master and slave are in blocking mode
     os.set_inheritable(slave, True)
     os.set_inheritable(master, False)
@@ -205,16 +206,20 @@ class Child:
         argv: Sequence[str],
         cwd: str,
         stdin: Optional[bytes] = None,
-        env: Optional[Dict[str, str]] = None,
+        env: Optional[dict[str, str]] = None,
         cwd_from: Optional['CwdRequest'] = None,
         is_clone_launch: str = '',
         add_listen_on_env_var: bool = True,
         hold: bool = False,
+        pass_fds: tuple[int, ...] = (),
+        remote_control_fd: int = -1,
     ):
         self.is_clone_launch = is_clone_launch
         self.id = next(child_counter)
         self.add_listen_on_env_var = add_listen_on_env_var
         self.argv = list(argv)
+        self.pass_fds = pass_fds
+        self.remote_control_fd = remote_control_fd
         if cwd_from:
             try:
                 cwd = cwd_from.modify_argv_for_launch_with_cwd(self.argv, env) or cwd
@@ -225,12 +230,12 @@ class Child:
         self.cwd = os.path.abspath(cwd)
         self.stdin = stdin
         self.env = env or {}
-        self.final_env:Dict[str, str] = {}
+        self.final_env:dict[str, str] = {}
         self.is_default_shell = bool(self.argv and self.argv[0] == shell_path)
         self.should_run_via_run_shell_kitten = is_macos and self.is_default_shell
         self.hold = hold
 
-    def get_final_env(self) -> Dict[str, str]:
+    def get_final_env(self) -> dict[str, str]:
         from kitty.options.utils import DELETE_ENV_VAR
         env = default_env().copy()
         opts = fast_data_types.get_options()
@@ -243,10 +248,13 @@ class Child:
         env['COLORTERM'] = 'truecolor'
         env['KITTY_PID'] = getpid()
         env['KITTY_PUBLIC_KEY'] = boss.encryption_public_key
-        if self.add_listen_on_env_var and boss.listening_on:
+        if self.remote_control_fd > -1:
+            env['KITTY_LISTEN_ON'] = f'fd:{self.remote_control_fd}'
+        elif self.add_listen_on_env_var and boss.listening_on:
             env['KITTY_LISTEN_ON'] = boss.listening_on
         else:
             env.pop('KITTY_LISTEN_ON', None)
+        env.pop('KITTY_STDIO_FORWARDED', None)
         if self.cwd:
             # needed in case cwd is a symlink, in which case shells
             # can use it to display the current directory name rather
@@ -259,8 +267,6 @@ class Child:
         elif opts.terminfo_type == 'direct':
             env['TERMINFO'] = base64_terminfo_data()
         env['KITTY_INSTALLATION_DIR'] = kitty_base_dir
-        if opts.forward_stdio:
-            env['KITTY_STDIO_FORWARDED'] = '3'
         self.unmodified_argv = list(self.argv)
         if not self.should_run_via_run_shell_kitten and 'disabled' not in opts.shell_integration:
             from .shell_integration import modify_shell_environ
@@ -292,6 +298,9 @@ class Child:
         self.final_env = self.get_final_env()
         argv = list(self.argv)
         cwd = self.cwd
+        pass_fds = self.pass_fds
+        if self.remote_control_fd > -1:
+            pass_fds += self.remote_control_fd,
         if self.should_run_via_run_shell_kitten:
             # bash will only source ~/.bash_profile if it detects it is a login
             # shell (see the invocation section of the bash man page), which it
@@ -312,12 +321,14 @@ class Child:
             if ksi == 'invalid':
                 ksi = 'enabled'
             argv = [kitten_exe(), 'run-shell', '--shell', shlex.join(argv), '--shell-integration', ksi]
-            if is_macos:
+            if is_macos and not pass_fds and not opts.forward_stdio:
                 # In addition for getlogin() to work we need to run the shell
                 # via the /usr/bin/login wrapper, sigh.
                 # And login on macOS looks for .hushlogin in CWD instead of
                 # HOME, bloody idiotic so we cant cwd when running it.
                 # https://github.com/kovidgoyal/kitty/issues/6511
+                # login closes inherited file descriptors so dont use it when
+                # forward_stdio or pass_fds are used.
                 import pwd
                 user = pwd.getpwuid(os.geteuid()).pw_name
                 if cwd:
@@ -332,7 +343,7 @@ class Child:
         env = tuple(f'{k}={v}' for k, v in self.final_env.items())
         pid = fast_data_types.spawn(
             final_exe, cwd, tuple(argv), env, master, slave, stdin_read_fd, stdin_write_fd,
-            ready_read_fd, ready_write_fd, tuple(handled_signals), kitten_exe(), opts.forward_stdio)
+            ready_read_fd, ready_write_fd, tuple(handled_signals), kitten_exe(), opts.forward_stdio, pass_fds)
         os.close(slave)
         self.pid = pid
         self.child_fd = master
@@ -363,7 +374,7 @@ class Child:
         os.close(self.terminal_ready_fd)
         self.terminal_ready_fd = -1
 
-    def cmdline_of_pid(self, pid: int) -> List[str]:
+    def cmdline_of_pid(self, pid: int) -> list[str]:
         try:
             ans = cmdline_of_pid(pid)
         except Exception:
@@ -373,7 +384,7 @@ class Child:
         return ans
 
     @property
-    def foreground_processes(self) -> List[ProcessDesc]:
+    def foreground_processes(self) -> list[ProcessDesc]:
         if self.child_fd is None:
             return []
         try:
@@ -393,7 +404,7 @@ class Child:
             return []
 
     @property
-    def cmdline(self) -> List[str]:
+    def cmdline(self) -> list[str]:
         try:
             assert self.pid is not None
             return self.cmdline_of_pid(self.pid) or list(self.argv)
@@ -401,7 +412,7 @@ class Child:
             return list(self.argv)
 
     @property
-    def foreground_cmdline(self) -> List[str]:
+    def foreground_cmdline(self) -> list[str]:
         try:
             assert self.pid_for_cwd is not None
             return self.cmdline_of_pid(self.pid_for_cwd) or self.cmdline
@@ -409,7 +420,7 @@ class Child:
             return self.cmdline
 
     @property
-    def environ(self) -> Dict[str, str]:
+    def environ(self) -> dict[str, str]:
         try:
             assert self.pid is not None
             return environ_of_process(self.pid) or self.final_env.copy()
@@ -467,7 +478,7 @@ class Child:
         return self.get_foreground_cwd()
 
     @property
-    def foreground_environ(self) -> Dict[str, str]:
+    def foreground_environ(self) -> dict[str, str]:
         pid = self.pid_for_cwd
         if pid is not None:
             with suppress(Exception):
