@@ -12,12 +12,14 @@ typedef enum {
     STATE_CLEANUP_FUNC,
     GLFW_CLEANUP_FUNC,
     DESKTOP_CLEANUP_FUNC,
-    FREETYPE_CLEANUP_FUNC,
     CORE_TEXT_CLEANUP_FUNC,
     COCOA_CLEANUP_FUNC,
     PNG_READER_CLEANUP_FUNC,
     FONTCONFIG_CLEANUP_FUNC,
+    FREETYPE_CLEANUP_FUNC,
     SYSTEMD_CLEANUP_FUNC,
+    SHADERS_CLEANUP_FUNC,
+    LINE_CLEANUP_FUNC,
 
     NUM_CLEANUP_FUNCS
 } AtExitCleanupFunc;
