@@ -4,7 +4,6 @@
 
 import inspect
 import os
-import pprint
 import re
 import textwrap
 from typing import Any, Callable, Dict, Iterator, List, Set, Tuple, Union, get_type_hints
@@ -279,6 +278,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
     a('')
     a('')
     a('defaults = Options()')
+    a('')
     for option_name, (typ, mval) in is_mutiple_vars.items():
         a(f'defaults.{option_name} = {mval[""]!r}')
         if mval['macos']:
@@ -290,6 +290,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
             a('if not is_macos:')
             a(f'    defaults.{option_name}.update({mval["linux"]!r}')
 
+    a('')
     for aname, func in action_parsers.items():
         a(f'defaults.{aname} = [')
         only: Dict[str, List[Tuple[str, Callable[..., Any]]]] = {}
@@ -302,8 +303,9 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
             else:
                 for val in func(text):
                     a(f'    # {sc.name}')
-                    a(f'    {val!r}, ')
+                    a(f'    {val!r},')
         a(']')
+        a('')
         if only:
             imports.add(('kitty.constants', 'is_macos'))
             for cond, items in only.items():
@@ -312,6 +314,7 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
                 for (text, parser_func) in items:
                     for val in parser_func(text):
                         a(f'    defaults.{aname}.append({val!r})')
+                a('')
 
     t('')
     t('')
@@ -385,8 +388,10 @@ def generate_class(defn: Definition, loc: str) -> Tuple[str, str]:
             a(f'{name} = {cdefn}')
 
     a('')
-    a('option_names = (  # {{''{')
-    a(' ' + pprint.pformat(tuple(sorted(option_names, key=natural_keys)))[1:] + '  # }}''}')
+    a('option_names = (')
+    for option_name in sorted(option_names, key=natural_keys):
+        a(f'    {option_name!r},')
+    a(')')
     class_def = '\n'.join(preamble + ['', ''] + class_lines)
 
     preamble = ['# generated by gen-config.py DO NOT edit', '']
@@ -437,15 +442,20 @@ def generate_c_conversion(loc: str, ctypes: List[Union[Option, MultiOption]]) ->
 
 def write_output(loc: str, defn: Definition, extra_after_type_defn: str = '') -> None:
     cls, tc = generate_class(defn, loc)
-    with open(os.path.join(*loc.split('.'), 'options', 'types.py'), 'w') as f:
-        f.write(f'{cls}\n')
-        f.write(extra_after_type_defn)
-    with open(os.path.join(*loc.split('.'), 'options', 'parse.py'), 'w') as f:
-        f.write(f'{tc}\n')
     ctypes = []
+    has_secret = []
     for opt in defn.root_group.iter_all_non_groups():
         if isinstance(opt, (Option, MultiOption)) and opt.ctype:
             ctypes.append(opt)
+        if getattr(opt, 'has_secret', False):
+            has_secret.append(opt.name)
+    with open(os.path.join(*loc.split('.'), 'options', 'types.py'), 'w') as f:
+        f.write(f'{cls}\n')
+        f.write(extra_after_type_defn)
+        if has_secret:
+            f.write('\n\nsecret_options = ' + repr(tuple(has_secret)))
+    with open(os.path.join(*loc.split('.'), 'options', 'parse.py'), 'w') as f:
+        f.write(f'{tc}\n')
     if ctypes:
         c = generate_c_conversion(loc, ctypes)
         with open(os.path.join(*loc.split('.'), 'options', 'to-c-generated.h'), 'w') as f:
