@@ -198,6 +198,12 @@ typedef struct {
     monotonic_t last_drag_scroll_at;
     uint32_t last_special_key_pressed;
     WindowBarData title_bar_data, url_target_bar_data;
+    id_type redirect_keys_to_overlay;
+    struct {
+        bool enabled;
+        void *key_data;
+        size_t count, capacity;
+    } buffered_keys;
     struct {
         PendingClick *clicks;
         size_t num, capacity;
@@ -417,3 +423,4 @@ void change_live_resize_state(OSWindow*, bool);
 bool render_os_window(OSWindow *w, monotonic_t now, bool ignore_render_frames, bool scan_for_animated_images);
 void update_mouse_pointer_shape(void);
 void adjust_window_size_for_csd(OSWindow *w, int width, int height, int *adjusted_width, int *adjusted_height);
+void dispatch_buffered_keys(Window *w);
