@@ -3,21 +3,17 @@
 
 import os
 import re
+from collections.abc import Sequence
 from functools import lru_cache, partial, wraps
 from string import Formatter as StringFormatter
 from typing import (
     Any,
     Callable,
-    Dict,
-    List,
     NamedTuple,
     Optional,
-    Sequence,
-    Tuple,
 )
 
 from .borders import Border, BorderColor
-from .config import build_ansi_color_table
 from .constants import config_dir
 from .fast_data_types import (
     DECAWM,
@@ -31,7 +27,9 @@ from .fast_data_types import (
     set_tab_bar_render_data,
     update_tab_bar_edge_colors,
     viewport_for_window,
+    wcswidth,
 )
+from .progress import ProgressState
 from .rgb import alpha_blend, color_as_sgr, color_from_int, to_color
 from .types import WindowGeometry, run_once
 from .typing import EdgeLiteral, PowerlineStyle
@@ -51,6 +49,9 @@ class TabBarData(NamedTuple):
     active_bg: Optional[int]
     inactive_fg: Optional[int]
     inactive_bg: Optional[int]
+    num_of_windows_with_progress: int
+    total_progress: int
+    last_focused_window_with_progress_id: int
 
 
 class DrawData(NamedTuple):
@@ -94,12 +95,12 @@ def as_rgb(x: int) -> int:
     return (x << 8) | 2
 
 
-@lru_cache()
+@lru_cache
 def report_template_failure(template: str, e: str) -> None:
     log_error(f'Invalid tab title template: "{template}" with error: {e}')
 
 
-@lru_cache()
+@lru_cache
 def compile_template(template: str) -> Any:
     try:
         return compile('f"""' + template + '"""', '<template>', 'eval')
@@ -122,6 +123,8 @@ class ColorFormatter:
         elif q == 'tab':
             col = color_from_int((self.draw_data.tab_bg if self.which == '4' else self.draw_data.tab_fg)(self.tab_data))
             ans = f'8{color_as_sgr(col)}'
+        elif q.startswith('color'):
+            ans = f'8:5:{int(q[5:])}'
         else:
             if name.startswith('_'):
                 q = f'#{name[1:]}'
@@ -143,7 +146,7 @@ class Formatter:
 
 
 @run_once
-def super_sub_maps() -> Tuple[Dict[int, int], Dict[int, int]]:
+def super_sub_maps() -> tuple[dict[int, int], dict[int, int]]:
     import string
     sup_table = str.maketrans(
         string.ascii_lowercase + string.ascii_uppercase + string.digits + '+-=()',
@@ -156,7 +159,7 @@ def super_sub_maps() -> Tuple[Dict[int, int], Dict[int, int]]:
 
 class SupSub:
 
-    def __init__(self, data: Dict[str, Any], is_subscript: bool = False):
+    def __init__(self, data: dict[str, Any], is_subscript: bool = False):
         self.__data = data
         self.__is_subscript = is_subscript
 
@@ -220,9 +223,40 @@ class TabAccessor:
         tab = get_boss().tab_for_id(self.tab_id)
         return os.path.basename((tab.get_exe_of_active_window(oldest=True) if tab else '') or '')
 
+    @property
+    def last_focused_progress_percent(self) -> str:
+        tab = get_boss().tab_for_id(self.tab_id)
+        if not tab or not tab.last_focused_window_with_progress_id:
+            return ''
+        w = get_boss().window_id_map.get(tab.last_focused_window_with_progress_id)
+        if w is None:
+            return ''
+        if w.progress.state is ProgressState.error:
+            return '  '
+        if w.progress.state is ProgressState.unset:
+            return ''
+        if w.progress.state is ProgressState.indeterminate:
+            return '  '
+        p = f'{w.progress.percent}% '
+        if w.progress.state is ProgressState.paused:
+            return f'  {p}'
+        return p
+
+    @property
+    def progress_percent(self) -> str:
+        tab = get_boss().tab_for_id(self.tab_id)
+        if not tab or not tab.last_focused_window_with_progress_id:
+            return ''
+        if tab.num_of_windows_with_progress <= 1:
+            return self.last_focused_progress_percent
+        p = int(tab.total_progress / tab.num_of_windows_with_progress)
+        return f'{p}% '
+
+
 
 safe_builtins = {
     'max': max, 'min': min, 'str': str, 'repr': repr, 'abs': abs, 'len': len, 'chr': chr, 'ord': ord, 're': re,
+    'wcswidth': wcswidth,
 }
 
 
@@ -385,7 +419,7 @@ def draw_tab_with_fade(
     return end
 
 
-powerline_symbols: Dict[PowerlineStyle, Tuple[str, str]] = {
+powerline_symbols: dict[PowerlineStyle, tuple[str, str]] = {
     'slanted': ('', '╱'),
     'round': ('', '')
 }
@@ -481,8 +515,8 @@ class TabBar:
         self.os_window_id = os_window_id
         self.num_tabs = 1
         self.data_buffer_size = 0
-        self.blank_rects: Tuple[Border, ...] = ()
-        self.cell_ranges: List[Tuple[int, int]] = []
+        self.blank_rects: tuple[Border, ...] = ()
+        self.cell_ranges: list[tuple[int, int]] = []
         self.laid_out_once = False
         self.apply_options()
 
@@ -495,11 +529,8 @@ class TabBar:
             self.screen = s = Screen(None, 1, 10, 0, self.cell_width, cell_height)
         else:
             s = self.screen
-        s.color_profile.update_ansi_color_table(build_ansi_color_table(opts))
-        s.color_profile.set_configured_colors(
-            color_as_int(opts.inactive_tab_foreground),
-            color_as_int(opts.tab_bar_background or opts.background)
-        )
+        s.color_profile.default_fg = opts.inactive_tab_foreground
+        s.color_profile.default_bg = opts.tab_bar_background or opts.background
         sep = opts.tab_separator
         self.trailing_spaces = self.leading_spaces = 0
         while sep and sep[0] == ' ':
@@ -543,7 +574,7 @@ class TabBar:
         else:
             self.align = lambda: None
 
-    def patch_colors(self, spec: Dict[str, Optional[int]]) -> None:
+    def patch_colors(self, spec: dict[str, Optional[int]]) -> None:
         opts = get_options()
         atf = spec.get('active_tab_foreground')
         if isinstance(atf, int):
@@ -575,11 +606,12 @@ class TabBar:
             ifg = color_from_int(fg)
             if ifg is not None:
                 self.draw_data = self.draw_data._replace(inactive_fg=ifg)
-        self.screen.color_profile.set_configured_colors(fg, bg)
-        self.screen.color_profile.update_ansi_color_table(build_ansi_color_table(opts))
+        self.screen.color_profile.reload_from_opts()
+        self.screen.color_profile.default_fg = color_from_int(fg)
+        self.screen.color_profile.default_bg = color_from_int(bg)
 
     @property
-    def current_colors(self) -> Dict[str, Color]:
+    def current_colors(self) -> dict[str, Color]:
         return {
             'active_tab_foreground': self.draw_data.active_fg,
             'inactive_tab_foreground': self.draw_data.inactive_fg,
@@ -590,7 +622,7 @@ class TabBar:
 
     def update_blank_rects(self, central: Region, tab_bar: Region, vw: int, vh: int) -> None:
         opts = get_options()
-        blank_rects: List[Border] = []
+        blank_rects: list[Border] = []
         bg = BorderColor.tab_bar_margin_color if opts.tab_bar_margin_color is not None else BorderColor.default_bg
         if opts.tab_bar_margin_height:
             if opts.tab_bar_edge == 3:  # bottom
@@ -605,7 +637,7 @@ class TabBar:
                     blank_rects.append(Border(0, tab_bar.bottom + 1, vw, central.top, bg))
         g = self.window_geometry
         left_bg = right_bg = bg
-        if opts.tab_bar_margin_color is None:
+        if opts.tab_bar_margin_color is None or opts.tab_bar_margin_width == 0:
             left_bg = BorderColor.tab_bar_left_edge_color
             right_bg = BorderColor.tab_bar_right_edge_color
         if g.left > 0:
@@ -638,7 +670,7 @@ class TabBar:
         last_tab = data[-1] if data else None
         ed = ExtraData()
 
-        def draw_tab(i: int, tab: TabBarData, cell_ranges: List[Tuple[int, int]], max_tab_length: int) -> None:
+        def draw_tab(i: int, tab: TabBarData, cell_ranges: list[tuple[int, int]], max_tab_length: int) -> None:
             ed.prev_tab = data[i - 1] if i > 0 else None
             ed.next_tab = data[i + 1] if i + 1 < len(data) else None
             s.cursor.bg = as_rgb(self.draw_data.tab_bg(t))
@@ -687,7 +719,7 @@ class TabBar:
 
         s.cursor.x = 0
         s.erase_in_line(2, False)
-        cr: List[Tuple[int, int]] = []
+        cr: list[tuple[int, int]] = []
         ed.for_layout = False
         for i, t in enumerate(data):
             try:
