@@ -20,3 +20,5 @@ typedef struct {
 void initialize_wcs_state(WCSState *state);
 int wcswidth_step(WCSState *state, const char_type ch);
 PyObject * wcswidth_std(PyObject UNUSED *self, PyObject *str);
+PyObject * unicode_database_version(PyObject UNUSED *self, PyObject *str);
+size_t wcswidth_string(const char_type *s);
