@@ -204,9 +204,9 @@ class TestDataTypes(BaseTest):
         l0.add_combining_char(0, '\u0302')
         self.ae(l0[0], ' \u0300\U000e0100\u0302')
         l0.add_combining_char(0, '\u0301')
-        self.ae(l0[0], ' \u0300\U000e0100\u0301')
+        self.ae(l0[0], ' \u0300\U000e0100\u0302\u0301')
         self.ae(l0[1], '\0')
-        self.ae(str(l0), ' \u0300\U000e0100\u0301')
+        self.ae(str(l0), ' \u0300\U000e0100\u0302\u0301')
         t = 'Testing with simple text'
         lb = LineBuf(2, len(t))
         l0 = lb.line(0)
@@ -543,8 +543,8 @@ class TestDataTypes(BaseTest):
         c.bg = (1 << 24) | (2 << 16) | (3 << 8) | 2
         c.decoration_fg = (5 << 8) | 1
         l2.set_text('1', 0, 1, c)
-        self.ae(l2.as_ansi(), '\x1b[1;2;3;7;9;34;48:2:1:2:3;58:5:5m' '1'
-                '\x1b[22;23;27;29;39;49;59m' '0000')
+        self.ae(str(l2), '10000')
+        self.ae(l2.as_ansi(), '\x1b[1;2;3;7;9;34;48:2:1:2:3;58:5:5m' '1' '\x1b[22;23;27;29;39;49;59m' '0000')  # ]]
         lb = filled_line_buf()
         for i in range(1, lb.ynum + 1):
             lb.set_continued(i, True)
@@ -560,9 +560,10 @@ class TestDataTypes(BaseTest):
         def q(x, y=''):
             self.ae(y or x, strip_csi(x))
         q('test')
-        q('a\x1bbc', 'ac')
+        q('a\x1bbc', 'abc')
         q('a\x1b[bc', 'ac')
         q('a\x1b[12;34:43mbc', 'abc')
+        q('a\x1b[12;34:43\U0001f638', 'a\U0001f638')
 
     def test_single_key(self):
         from kitty.fast_data_types import GLFW_MOD_KITTY, GLFW_MOD_SHIFT, SingleKey
