@@ -1,37 +1,44 @@
 #!/usr/bin/env python
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
-import time
-from base64 import standard_b64encode
 from binascii import hexlify
 from functools import partial
 
-from kitty.fast_data_types import CURSOR_BLOCK, base64_decode, base64_encode, parse_bytes, parse_bytes_dump
-from kitty.notify import NotificationCommand, handle_notification_cmd, notification_activated, reset_registry
+from kitty.fast_data_types import (
+    CURSOR_BLOCK,
+    VT_PARSER_BUFFER_SIZE,
+    base64_decode,
+    base64_encode,
+    has_avx2,
+    has_sse4_2,
+    test_find_either_of_two_bytes,
+    test_utf8_decode_to_sentinel,
+)
 
-from . import BaseTest
+from . import BaseTest, parse_bytes
+
+
+def cnv(x):
+    if isinstance(x, memoryview):
+        x = str(x, 'utf-8')
+    return x
 
 
 class CmdDump(list):
 
-    def __call__(self, *a):
-        self.append(a)
+    def __call__(self, window_id, *a):
+        if a and a[0] == 'bytes':
+            return
+        if a and a[0] == 'error':
+            a = a[1:]
+        self.append(tuple(map(cnv, a)))
 
-
-class TestParser(BaseTest):
-
-    def parse_bytes_dump(self, s, x, *cmds):
-        cd = CmdDump()
-        if isinstance(x, str):
-            x = x.encode('utf-8')
-        cmds = tuple(('draw', x) if isinstance(x, str) else x for x in cmds)
-        parse_bytes_dump(cd, s, x)
+    def get_result(self):
         current = ''
         q = []
-        for args in cd:
+        for args in self:
             if args[0] == 'draw':
-                if args[1] is not None:
-                    current += args[1]
+                current += args[1]
             else:
                 if current:
                     q.append(('draw', current))
@@ -39,7 +46,106 @@ class TestParser(BaseTest):
                 q.append(args)
         if current:
             q.append(('draw', current))
-        self.ae(tuple(q), cmds)
+        return tuple(q)
+
+
+class TestParser(BaseTest):
+
+    def create_write_buffer(self, screen):
+        return screen.test_create_write_buffer()
+
+    def write_bytes(self, screen, write_buf, data):
+        if isinstance(data, str):
+            data = data.encode('utf-8')
+        s = screen.test_commit_write_buffer(data, write_buf)
+        return data[s:]
+
+    def parse_written_data(self, screen, *cmds):
+        cd = CmdDump()
+        screen.test_parse_written_data(cd)
+        cmds = tuple(('draw', x) if isinstance(x, str) else tuple(map(cnv, x)) for x in cmds)
+        self.ae(cmds, cd.get_result())
+
+    def parse_bytes_dump(self, s, x, *cmds):
+        cd = CmdDump()
+        if isinstance(x, str):
+            x = x.encode('utf-8')
+        cmds = tuple(('draw', x) if isinstance(x, str) else tuple(map(cnv, x)) for x in cmds)
+        parse_bytes(s, x, cd)
+        self.ae(cmds, cd.get_result())
+
+    def test_charsets(self):
+        s = self.create_screen()
+        pb = partial(self.parse_bytes_dump, s)
+        pb(b'\xc3')
+        pb(b'\xa1', ('draw', b'\xc3\xa1'.decode('utf-8')))
+        s = self.create_screen()
+        pb = partial(self.parse_bytes_dump, s)
+        pb('\033)0\x0e/_', ('screen_designate_charset', 1, ord('0')), ('screen_change_charset', 1), '/_')
+        self.ae(str(s.line(0)), '/\xa0')
+        s = self.create_screen()
+        pb = partial(self.parse_bytes_dump, s)
+        pb('\033(0/_', ('screen_designate_charset', 0, ord('0')), '/_')
+        self.ae(str(s.line(0)), '/\xa0')
+
+    def test_parser_threading(self):
+        s = self.create_screen()
+
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), 'a\x1b]2;some title'))
+        b = self.create_write_buffer(s)
+        self.parse_written_data(s, 'a')
+        self.assertFalse(self.write_bytes(s, b, ' full\x1b\\'))
+        self.parse_written_data(s, ('set_title', 'some title full'))
+
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), 'a\x1b]'))
+        b = self.create_write_buffer(s)
+        self.parse_written_data(s, 'a')
+        self.assertFalse(self.write_bytes(s, b, '2;title\x1b\\'))
+        self.parse_written_data(s, ('set_title', 'title'))
+
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), 'a\x1b'))
+        b = self.create_write_buffer(s)
+        self.parse_written_data(s, 'a')
+        self.assertFalse(self.write_bytes(s, b, ']2;title\x1b\\'))
+        self.parse_written_data(s, ('set_title', 'title'))
+
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), 'a\x1b]2;some title\x1b'))
+        b = self.create_write_buffer(s)
+        self.parse_written_data(s, 'a')
+        self.assertFalse(self.write_bytes(s, b, '\\b'))
+        self.parse_written_data(s, ('set_title', 'some title'), 'b')
+
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), '1\x1b'))
+        self.parse_written_data(s, '1')
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), 'E2'))
+        self.parse_written_data(s, ('screen_nel',), ('draw', '2'))
+
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), '1\x1b[2'))
+        self.parse_written_data(s, '1')
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), '3mx'))
+        self.parse_written_data(s, ('select_graphic_rendition', '23'), 'x')
+
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), '1\x1b'))
+        self.parse_written_data(s, '1')
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), '[23mx'))
+        self.parse_written_data(s, ('select_graphic_rendition', '23'), 'x')
+
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), '1\x1b['))
+        self.parse_written_data(s, '1')
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), '23mx'))
+        self.parse_written_data(s, ('select_graphic_rendition', '23'), 'x')
+
+        # test full write
+        sz = VT_PARSER_BUFFER_SIZE // 3 + 7
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), b'a' * sz))
+        self.assertFalse(self.write_bytes(s, self.create_write_buffer(s), b'b' * sz))
+        left = self.write_bytes(s, self.create_write_buffer(s), b'c' * sz)
+        self.assertTrue(len(left), 3 * sz - VT_PARSER_BUFFER_SIZE)
+        self.assertFalse(self.create_write_buffer(s))
+        s.test_parse_written_data()
+        b = self.create_write_buffer(s)
+        self.assertTrue(b)
+        self.write_bytes(s, b, b'')
 
     def test_base64(self):
         for src, expected in {
@@ -65,12 +171,172 @@ class TestParser(BaseTest):
         self.ae(str(s.line(1)), '6')
         self.ae(str(s.line(2)), ' 123')
         self.ae(str(s.line(3)), '45')
-        parse_bytes(s, b'\rabcde')
+        pb(b'\rabcde', ('screen_carriage_return',), 'abcde')
         self.ae(str(s.line(3)), 'abcde')
         pb('\rßxyz1', ('screen_carriage_return',), 'ßxyz1')
         self.ae(str(s.line(3)), 'ßxyz1')
         pb('ニチ ', 'ニチ ')
         self.ae(str(s.line(4)), 'ニチ ')
+        s.reset()
+        self.assertFalse(str(s.line(1)) + str(s.line(2)) + str(s.line(3)))
+        c1_controls = '\x84\x85\x88\x8d\x8e\x8f\x90\x96\x97\x98\x9a\x9b\x9c\x9d\x9e\x9f'
+        pb(c1_controls, c1_controls)
+        self.assertFalse(str(s.line(1)) + str(s.line(2)) + str(s.line(3)))
+        pb('😀'.encode()[:-1])
+        pb('\x1b\x1b%a', '\ufffd', ('Unknown char after ESC: 0x1b',), ('draw', '%a'))
+
+    def test_utf8_parsing(self):
+        s = self.create_screen()
+        pb = partial(self.parse_bytes_dump, s)
+        pb(b'"\xbf"', '"\ufffd"')
+        pb(b'"\x80"', '"\ufffd"')
+        pb(b'"\x80\xbf"', '"\ufffd\ufffd"')
+        pb(b'"\x80\xbf\x80"', '"\ufffd\ufffd\ufffd"')
+        pb(b'"\xc0 "', '"\ufffd "')
+        pb(b'"\xfe"', '"\ufffd"')
+        pb(b'"\xff"', '"\ufffd"')
+        pb(b'"\xff\xfe"', '"\ufffd\ufffd"')
+        pb(b'"\xfe\xfe\xff\xff"', '"\ufffd\ufffd\ufffd\ufffd"')
+        pb(b'"\xef\xbf"', '"\ufffd"')
+        pb(b'"\xe0\xa0"', '"\ufffd"')
+        pb(b'"\xf0\x9f\x98"', '"\ufffd"')
+
+    def test_utf8_simd_decode(self):
+        def unsupported(which):
+            return (which == 2 and not has_sse4_2) or (which == 3 and not has_avx2)
+
+        def reset_state():
+            test_utf8_decode_to_sentinel(b'', -1)
+
+        def asbytes(x):
+            if isinstance(x, str):
+                x = x.encode()
+            return x
+
+        def t(*a, which=2):
+            if unsupported(which):
+                return
+
+            def parse_parts(which):
+                total_consumed = 0
+                esc_found = False
+                parts = []
+                for x in a:
+                    found_sentinel, x, num_consumed = test_utf8_decode_to_sentinel(asbytes(x), which)
+                    total_consumed += num_consumed
+                    if found_sentinel:
+                        esc_found = found_sentinel
+                    parts.append(x)
+                return esc_found, ''.join(parts), total_consumed
+
+            reset_state()
+            actual = parse_parts(1)
+            reset_state()
+            expected = parse_parts(which)
+            self.ae(expected, actual, msg=f'Failed for {a} with {which=}\n{expected!r} !=\n{actual!r}')
+            return actual
+
+        def double_test(x):
+            for which in (2, 3):
+                t(x, which=which)
+            t(x*2, which=3)
+            reset_state()
+
+        # incomplete trailer at end of vector
+        t("a"*10 + "😸😸" + "b"*15)
+
+        x = double_test
+        x('2:α3')
+        x('2:α\x1b3')
+        x('2:α3:≤4:😸|')
+        x('abcd1234efgh5678')
+        x('abc\x1bd1234efgh5678')
+        x('abcd1234efgh5678ijklABCDmnopEFGH')
+
+        for which in (2, 3):
+            x = partial(t, which=which)
+            x('abcdef', 'ghijk')
+            x('2:α3', ':≤4:😸|')
+            # trailing incomplete sequence
+            for prefix in (b'abcd', '😸'.encode()):
+                for suffix in (b'1234', '😸'.encode()):
+                    x(prefix + b'\xf0\x9f', b'\x98\xb8' + suffix)
+                    x(prefix + b'\xf0\x9f\x9b', b'\xb8' + suffix)
+                    x(prefix + b'\xf0', b'\x9f\x98\xb8' + suffix)
+                    x(prefix + b'\xc3', b'\xa4' + suffix)
+                    x(prefix + b'\xe2', b'\x89\xa4' + suffix)
+                    x(prefix + b'\xe2\x89', b'\xa4' + suffix)
+
+        def test_expected(src, expected, which=2):
+            if unsupported(which):
+                return
+            reset_state()
+            _, actual, _ = t(b'filler' + asbytes(src), which=which)
+            expected = 'filler' + expected
+            self.ae(expected, actual, f'Failed for: {src!r} with {which=}')
+
+        for which in (1, 2, 3):
+            pb = partial(test_expected, which=which)
+            pb('ニチ', 'ニチ')
+            pb('\x84\x85', '\x84\x85')
+            pb('\x84\x85', '\x84\x85')
+            pb('\uf4df', '\uf4df')
+            pb('\uffff', '\uffff')
+            pb('\0', '\0')
+            pb(chr(0x10ffff), chr(0x10ffff))
+            # various invalid input
+            pb(b'abcd\xf51234', 'abcd\ufffd1234')  # bytes > 0xf4
+            pb(b'abcd\xff1234', 'abcd\ufffd1234')  # bytes > 0xf4
+            pb(b'"\xbf"', '"\ufffd"')
+            pb(b'"\x80"', '"\ufffd"')
+            pb(b'"\x80\xbf"', '"\ufffd\ufffd"')
+            pb(b'"\x80\xbf\x80"', '"\ufffd\ufffd\ufffd"')
+            pb(b'"\xc0 "', '"\ufffd "')
+            pb(b'"\xfe"', '"\ufffd"')
+            pb(b'"\xff"', '"\ufffd"')
+            pb(b'"\xff\xfe"', '"\ufffd\ufffd"')
+            pb(b'"\xfe\xfe\xff\xff"', '"\ufffd\ufffd\ufffd\ufffd"')
+            pb(b'"\xef\xbf"', '"\ufffd"')
+            pb(b'"\xe0\xa0"', '"\ufffd"')
+            pb(b'"\xf0\x9f\x98"', '"\ufffd"')
+            pb(b'"\xef\x93\x94\x95"', '"\uf4d4\ufffd"')
+
+    def test_find_either_of_two_bytes(self):
+        sizes = []
+        if has_sse4_2:
+            sizes.append(2)
+        if has_avx2:
+            sizes.append(3)
+        sizes.append(0)
+
+        def test(buf, a, b, align_offset=0):
+            a_, b_ = ord(a), ord(b)
+            expected = test_find_either_of_two_bytes(buf, a_, b_, 1, 0)
+            for sz in sizes:
+                actual = test_find_either_of_two_bytes(buf, a_, b_, sz, align_offset)
+                self.ae(expected, actual, f'Failed for: {buf!r} {a=} {b=} at {sz=} and {align_offset=}')
+
+        q = 'abc'
+        for off in range(32):
+            test(q, '<', '>', off)
+            test(q, ' ', 'b', off)
+            test(q, '<', 'a', off)
+            test(q, '<', 'b', off)
+            test(q, 'c', '>', off)
+
+        def tests(buf, a, b):
+            for sz in (0, 16, 32, 64, 79):
+                buf = (' ' * sz) + buf
+                for align_offset in range(32):
+                    test(buf, a, b, align_offset)
+        tests("", '<', '>')
+        tests("a", '\0', '\0')
+        tests("a", '<', '>')
+        tests("dsdfsfa", '1', 'a')
+        tests("xa", 'a', 'a')
+        tests("bbb", 'a', '1')
+        tests("bba", 'a', '<')
+        tests("baa", '>', 'a')
 
     def test_esc_codes(self):
         s = self.create_screen()
@@ -78,26 +344,10 @@ class TestParser(BaseTest):
         pb('12\033Da', '12', ('screen_index',), 'a')
         self.ae(str(s.line(0)), '12')
         self.ae(str(s.line(1)), '  a')
-        pb('\033x', ('Unknown char after ESC: 0x%x' % ord('x'),))
+        pb('\033xa', ('Unknown char after ESC: 0x%x' % ord('x'),), 'a')
         pb('\033c123', ('screen_reset', ), '123')
         self.ae(str(s.line(0)), '123')
-
-    def test_charsets(self):
-        s = self.create_screen()
-        pb = partial(self.parse_bytes_dump, s)
-        pb(b'\xc3')
-        pb(b'\xa1', ('draw', b'\xc3\xa1'.decode('utf-8')))
-        s = self.create_screen()
-        pb = partial(self.parse_bytes_dump, s)
-        pb('\033)0\x0e/_', ('screen_designate_charset', 1, ord('0')), ('screen_change_charset', 1), '/_')
-        self.ae(str(s.line(0)), '/\xa0')
-        self.assertTrue(s.callbacks.iutf8)
-        pb('\033%@_', ('screen_use_latin1', 1), '_')
-        self.assertFalse(s.callbacks.iutf8)
-        s = self.create_screen()
-        pb = partial(self.parse_bytes_dump, s)
-        pb('\033(0/_', ('screen_designate_charset', 0, ord('0')), '/_')
-        self.ae(str(s.line(0)), '/\xa0')
+        pb('\033.\033a', ('Unhandled charset related escape code: 0x2e 0x1b',), 'a')
 
     def test_csi_codes(self):
         s = self.create_screen()
@@ -108,8 +358,8 @@ class TestParser(BaseTest):
         self.ae(str(s.line(0)), 'xy bc')
         pb('x\033[2;7@y', 'x', ('CSI code @ has 2 > 1 parameters',), 'y')
         pb('x\033[2;-7@y', 'x', ('CSI code @ has 2 > 1 parameters',), 'y')
-        pb('x\033[-2@y', 'x', ('CSI code @ is not allowed to have negative parameter (-2)',), 'y')
-        pb('x\033[2-3@y', 'x', ('CSI code can contain hyphens only at the start of numbers',), 'y')
+        pb('x\033[-0001234567890@y', 'x', ('CSI code @ is not allowed to have negative parameter (-1234567890)',), 'y')
+        pb('x\033[2-3@y', 'x', ('Invalid character in CSI: 3 (0x33), ignoring the sequence',), '@y')
         pb('x\033[@y', 'x', ('screen_insert_characters', 1), 'y')
         pb('x\033[345@y', 'x', ('screen_insert_characters', 345), 'y')
         pb('x\033[345;@y', 'x', ('screen_insert_characters', 345), 'y')
@@ -121,6 +371,7 @@ class TestParser(BaseTest):
         pb('\033[3;2;H', ('screen_cursor_position', 3, 2))
         pb('\033[00000000003;0000000000000002H', ('screen_cursor_position', 3, 2))
         self.ae(s.cursor.x, 1), self.ae(s.cursor.y, 2)
+        pb('\033[0001234567890H', ('screen_cursor_position', 1234567890, 1))
         pb('\033[J', ('screen_erase_in_display', 0, 0))
         pb('\033[?J', ('screen_erase_in_display', 0, 1))
         pb('\033[?2J', ('screen_erase_in_display', 2, 1))
@@ -131,31 +382,36 @@ class TestParser(BaseTest):
         pb('\033[=c', ('report_device_attributes', 0, 61))
         s.reset()
 
-        def sgr(params):
-            return (('select_graphic_rendition', f'{x} ') for x in params.split())
+        def sgr(*params):
+            return (('select_graphic_rendition', f'{x}') for x in params)
 
         pb('\033[1;2;3;4;7;9;34;44m', *sgr('1 2 3 4 7 9 34 44'))
         for attr in 'bold italic reverse strikethrough dim'.split():
-            self.assertTrue(getattr(s.cursor, attr))
+            self.assertTrue(getattr(s.cursor, attr), attr)
         self.ae(s.cursor.decoration, 1)
         self.ae(s.cursor.fg, 4 << 8 | 1)
         self.ae(s.cursor.bg, 4 << 8 | 1)
-        pb('\033[38;5;1;48;5;7m', ('select_graphic_rendition', '38 5 1 '), ('select_graphic_rendition', '48 5 7 '))
+        pb('\033[38;5;1;48;5;7m', ('select_graphic_rendition', '38:5:1'), ('select_graphic_rendition', '48:5:7'))
         self.ae(s.cursor.fg, 1 << 8 | 1)
         self.ae(s.cursor.bg, 7 << 8 | 1)
-        pb('\033[38;2;1;2;3;48;2;7;8;9m', ('select_graphic_rendition', '38 2 1 2 3 '), ('select_graphic_rendition', '48 2 7 8 9 '))
+        pb('\033[38;2;1;2;3;48;2;7;8;9m', ('select_graphic_rendition', '38:2:1:2:3'), ('select_graphic_rendition', '48:2:7:8:9'))
         self.ae(s.cursor.fg, 1 << 24 | 2 << 16 | 3 << 8 | 2)
         self.ae(s.cursor.bg, 7 << 24 | 8 << 16 | 9 << 8 | 2)
         pb('\033[0;2m', *sgr('0 2'))
         pb('\033[;2m', *sgr('0 2'))
-        pb('\033[m', *sgr('0 '))
+        pb('\033[m', *sgr('0'))
         pb('\033[1;;2m', *sgr('1 0 2'))
-        pb('\033[38;5;1m', ('select_graphic_rendition', '38 5 1 '))
-        pb('\033[58;2;1;2;3m', ('select_graphic_rendition', '58 2 1 2 3 '))
-        pb('\033[38;2;1;2;3m', ('select_graphic_rendition', '38 2 1 2 3 '))
-        pb('\033[1001:2:1:2:3m', ('select_graphic_rendition', '1001 2 1 2 3 '))
+        pb('\033[38;5;1m', ('select_graphic_rendition', '38:5:1'))
+        pb('\033[58;2;1;2;3m', ('select_graphic_rendition', '58:2:1:2:3'))
+        pb('\033[38;2;1;2;3m', ('select_graphic_rendition', '38:2:1:2:3'))
+        pb('\033[1001:2:1:2:3m', ('select_graphic_rendition', '1001:2:1:2:3'))
         pb('\033[38:2:1:2:3;48:5:9;58;5;7m', (
-            'select_graphic_rendition', '38 2 1 2 3 '), ('select_graphic_rendition', '48 5 9 '), ('select_graphic_rendition', '58 5 7 '))
+            'select_graphic_rendition', '38:2:1:2:3'), ('select_graphic_rendition', '48:5:9'), ('select_graphic_rendition', '58:5:7'))
+        s.reset()
+        pb('\033[1;2;3;4:5;7;9;34;44m', *sgr('1 2 3', '4:5', '7 9 34 44'))
+        for attr in 'bold italic reverse strikethrough dim'.split():
+            self.assertTrue(getattr(s.cursor, attr), attr)
+        self.ae(s.cursor.decoration, 5)
         c = s.callbacks
         pb('\033[5n', ('report_device_status', 5, 0))
         self.ae(c.wtcbuf, b'\033[0n')
@@ -194,6 +450,18 @@ class TestParser(BaseTest):
         pb('\033[T', ('screen_reverse_scroll', 1))
         pb('\033[+T', ('screen_reverse_scroll_and_fill_from_scrollback', 1))
 
+        c.clear()
+        pb('\033[?2026$p', ('report_mode_status', 2026, 1))
+        self.ae(c.wtcbuf, b'\x1b[?2026;2$y')
+        c.clear()
+        pb('\033[?2026h', ('screen_set_mode', 2026, 1))
+        pb('\033[?2026$p', ('report_mode_status', 2026, 1))
+        self.ae(c.wtcbuf, b'\x1b[?2026;1$y')
+        pb('\033[?2026l', ('screen_reset_mode', 2026, 1))
+        c.clear()
+        pb('\033[?2026$p', ('report_mode_status', 2026, 1))
+        self.ae(c.wtcbuf, b'\x1b[?2026;2$y')
+
     def test_csi_code_rep(self):
         s = self.create_screen(8)
         pb = partial(self.parse_bytes_dump, s)
@@ -218,13 +486,13 @@ class TestParser(BaseTest):
         s = self.create_screen()
         pb = partial(self.parse_bytes_dump, s)
         c = s.callbacks
-        pb('a\033]2;x\\ryz\x9cbcde', 'a', ('set_title', 'x\\ryz'), 'bcde')
+        pb('a\033]2;x\\ryz\033\\bcde', 'a', ('set_title', 'x\\ryz'), 'bcde')
         self.ae(str(s.line(0)), 'abcde')
         self.ae(c.titlebuf, ['x\\ryz'])
         c.clear()
         pb('\033]\x07', ('set_title', ''), ('set_icon', ''))
         self.ae(c.titlebuf, ['']), self.ae(c.iconbuf, '')
-        pb('\033]ab\x07', ('set_title', 'ab'), ('set_icon', 'ab'))
+        pb('1\033]ab\x072', '1', ('set_title', 'ab'), ('set_icon', 'ab'), '2')
         self.ae(c.titlebuf, ['', 'ab']), self.ae(c.iconbuf, 'ab')
         c.clear()
         pb('\033]2;;;;\x07', ('set_title', ';;;'))
@@ -236,9 +504,9 @@ class TestParser(BaseTest):
         self.ae(c.colorbuf, '')
         c.clear()
         pb('\033]9;\x07', ('desktop_notify', 9, ''))
-        pb('\033]9;test it\x07', ('desktop_notify', 9, 'test it'))
+        pb('\033]9;test it with a nice long string\x07', ('desktop_notify', 9, 'test it with a nice long string'))
         pb('\033]99;moo=foo;test it\x07', ('desktop_notify', 99, 'moo=foo;test it'))
-        self.ae(c.notifications, [(9, ''), (9, 'test it'), (99, 'moo=foo;test it')])
+        self.ae(c.notifications, [(9, ''), (9, 'test it with a nice long string'), (99, 'moo=foo;test it')])
         c.clear()
         pb('\033]8;;\x07', ('set_active_hyperlink', None, None))
         pb('\033]8moo\x07', ('Ignoring malformed OSC 8 code',))
@@ -251,75 +519,12 @@ class TestParser(BaseTest):
         c.clear()
         pb('\033]52;p;xyz\x07', ('clipboard_control', 52, 'p;xyz'))
 
-    def test_desktop_notify(self):
-        reset_registry()
-        notifications = []
-        activations = []
-        prev_cmd = NotificationCommand()
-
-        def reset():
-            nonlocal prev_cmd
-            reset_registry()
-            del notifications[:]
-            del activations[:]
-            prev_cmd = NotificationCommand()
-
-        def notify(title, body, identifier):
-            notifications.append((title, body, identifier))
-
-        def h(raw_data, osc_code=99, window_id=1):
-            nonlocal prev_cmd
-            x = handle_notification_cmd(osc_code, raw_data, window_id, prev_cmd, notify)
-            if x is not None and osc_code == 99:
-                prev_cmd = x
-
-        def activated(identifier, window_id, focus, report):
-            activations.append((identifier, window_id, focus, report))
-
-        h('test it', osc_code=9)
-        self.ae(notifications, [('test it', '', 'i0')])
-        notification_activated(notifications[-1][-1], activated)
-        self.ae(activations, [('0', 1, True, False)])
-        reset()
-
-        h('d=0:i=x;title')
-        h('d=1:i=x:p=body;body')
-        self.ae(notifications, [('title', 'body', 'i0')])
-        notification_activated(notifications[-1][-1], activated)
-        self.ae(activations, [('x', 1, True, False)])
-        reset()
-
-        h('i=x:p=body:a=-focus;body')
-        self.ae(notifications, [('body', '', 'i0')])
-        notification_activated(notifications[-1][-1], activated)
-        self.ae(activations, [])
-        reset()
-
-        h('i=x:e=1;' + standard_b64encode(b'title').decode('ascii'))
-        self.ae(notifications, [('title', '', 'i0')])
-        notification_activated(notifications[-1][-1], activated)
-        self.ae(activations, [('x', 1, True, False)])
-        reset()
-
-        h('d=0:i=x:a=-report;title')
-        h('d=1:i=x:a=report;body')
-        self.ae(notifications, [('titlebody', '', 'i0')])
-        notification_activated(notifications[-1][-1], activated)
-        self.ae(activations, [('x', 1, True, True)])
-        reset()
-
-        h(';title')
-        self.ae(notifications, [('title', '', 'i0')])
-        notification_activated(notifications[-1][-1], activated)
-        self.ae(activations, [('0', 1, True, False)])
-        reset()
-
     def test_dcs_codes(self):
         s = self.create_screen()
         c = s.callbacks
         pb = partial(self.parse_bytes_dump, s)
         q = hexlify(b'kind').decode('ascii')
-        pb(f'a\033P+q{q}\x9cbcde', 'a', ('screen_request_capabilities', 43, q), 'bcde')
+        pb(f'a\033P+q{q}\033\\bcde', 'a', ('screen_request_capabilities', 43, q), 'bcde')
         self.ae(str(s.line(0)), 'abcde')
         self.ae(c.wtcbuf, '1+r{}={}'.format(q, '1b5b313b3242').encode('ascii'))
         c.clear()
@@ -337,75 +542,30 @@ class TestParser(BaseTest):
         c.clear()
         pb('\033P$qr\033\\', ('screen_request_capabilities', ord('$'), 'r'))
         self.ae(c.wtcbuf, f'\033P1$r{s.margin_top + 1};{s.margin_bottom + 1}r\033\\'.encode('ascii'))
+        pb('\033P@kitty-cmd{abc\033\\', ('handle_remote_cmd', '{abc'))
+        p = base64_encode('abcd').decode()
+        pb(f'\033P@kitty-print|{p}\033\\', ('handle_remote_print', p))
+        self.ae(['abcd'], s.callbacks.printbuf)
 
-    def test_sc81t(self):
-        s = self.create_screen()
-        pb = partial(self.parse_bytes_dump, s)
-        pb('\033 G', ('screen_set_8bit_controls', 1))
-        c = s.callbacks
-        pb('\033P$qm\033\\', ('screen_request_capabilities', ord('$'), 'm'))
-        self.ae(c.wtcbuf, b'\x901$rm\x9c')
         c.clear()
-        pb('\033[0c', ('report_device_attributes', 0, 0))
-        self.ae(c.wtcbuf, b'\x9b?62;c')
-
-    def test_pending(self):
-        s = self.create_screen()
-        timeout = 0.1
-        s.set_pending_timeout(timeout)
-        pb = partial(self.parse_bytes_dump, s)
-
+        pb('\033[?2026$p', ('report_mode_status', 2026, 1))
+        self.ae(c.wtcbuf, b'\x1b[?2026;2$y')
         pb('\033P=1s\033\\', ('screen_start_pending_mode',))
-        pb('a')
-        self.ae(str(s.line(0)), '')
-        pb('\033P=2s\033\\', ('draw', 'a'), ('screen_stop_pending_mode',))
-        self.ae(str(s.line(0)), 'a')
-        pb('\033P=1s\033\\', ('screen_start_pending_mode',))
-        pb('b')
-        self.ae(str(s.line(0)), 'a')
-        time.sleep(timeout)
-        pb('c', ('draw', 'bc'))
-        self.ae(str(s.line(0)), 'abc')
-        pb('\033P=1s\033\\d', ('screen_start_pending_mode',))
-        pb('\033P=2s\033\\', ('draw', 'd'), ('screen_stop_pending_mode',))
-        pb('\033P=1s\033\\e', ('screen_start_pending_mode',))
-        pb('\033P'), pb('='), pb('2s')
-        pb('\033\\', ('draw', 'e'), ('screen_stop_pending_mode',))
-        pb('\033P=1sxyz;.;\033\\''\033P=2skjf".,><?_+)98\033\\', ('screen_start_pending_mode',))
-        pb('\033P=1s\033\\f\033P=1s\033\\', ('screen_start_pending_mode',), ('screen_start_pending_mode',))
-        pb('\033P=2s\033\\', ('draw', 'f'), ('screen_stop_pending_mode',))
-        pb('\033P=1s\033\\XXX\033P=2s\033\\', ('screen_start_pending_mode',), ('draw', 'XXX'), ('screen_stop_pending_mode',))
+        c.clear()
+        pb('\033[?2026$p', ('report_mode_status', 2026, 1))
+        self.ae(c.wtcbuf, b'\x1b[?2026;1$y')
+        pb('\033P=2s\033\\', ('screen_stop_pending_mode',))
+        c.clear()
+        pb('\033[?2026$p', ('report_mode_status', 2026, 1))
+        self.ae(c.wtcbuf, b'\x1b[?2026;2$y')
 
-        pb('\033[?2026hXXX\033[?2026l', ('screen_set_mode', 2026, 1), ('draw', 'XXX'), ('screen_reset_mode', 2026, 1))
-        pb('\033[?2026h\033[32ma\033[?2026l', ('screen_set_mode', 2026, 1), ('select_graphic_rendition', '32 '), ('draw', 'a'), ('screen_reset_mode', 2026, 1))
-        pb('\033[?2026h\033P+q544e\033\\ama\033P=2s\033\\',
-           ('screen_set_mode', 2026, 1), ('screen_request_capabilities', 43, '544e'), ('draw', 'ama'), ('screen_stop_pending_mode',))
-        pb('\033P=1s\033\\\033(B\033P=2s\033\\', ('screen_start_pending_mode',), ('screen_designate_charset', 0, 66), ('screen_stop_pending_mode',))
-
-        s.reset()
-        s.set_pending_timeout(timeout)
-        pb('\033[?2026h', ('screen_set_mode', 2026, 1),)
-        pb('\033P+q')
-        time.sleep(1.2 * timeout)
-        pb(
-            '544e' + '\033\\\033P=2s\033\\',
-            ('screen_request_capabilities', 43, '544e'),
-            ('Pending mode stop command issued while not in pending mode, this can be '
-             'either a bug in the terminal application or caused by a timeout with no '
-             'data received for too long or by too much data in pending mode',),
-            ('screen_stop_pending_mode',)
-        )
-        self.assertEqual(str(s.line(0)), '')
 
     def test_oth_codes(self):
         s = self.create_screen()
         pb = partial(self.parse_bytes_dump, s)
-        for prefix in '\033_', '\u009f':
-            for suffix in '\u009c', '\033\\':
-                pb(f'a{prefix}+\\++{suffix}bcde', ('draw', 'a'), ('Unrecognized APC code: 0x2b',), ('draw', 'bcde'))
-        for prefix in '\033^', '\u009e':
-            for suffix in '\u009c', '\033\\':
-                pb(f'a{prefix}+\\++{suffix}bcde', ('draw', 'a'), ('Unrecognized PM code: 0x2b',), ('draw', 'bcde'))
+        pb('a\033_+\\+\033\\bcde', ('draw', 'a'), ('Unrecognized APC code: 0x2b',), ('draw', 'bcde'))
+        pb('a\033^+\\+\033\\bcde', ('draw', 'a'), ('Unrecognized PM code: 0x2b',), ('draw', 'bcde'))
+        pb('a\033X+\\+\033\\bcde', ('draw', 'a'), ('Unrecognized SOS code: 0x2b',), ('draw', 'bcde'))
 
     def test_graphics_command(self):
         from base64 import standard_b64encode
@@ -456,9 +616,9 @@ class TestParser(BaseTest):
     def test_deccara(self):
         s = self.create_screen()
         pb = partial(self.parse_bytes_dump, s)
-        pb('\033[$r', ('deccara', '0 0 0 0 0 '))
+        pb('\033[$r', ('deccara', '0 0 0 0 0'))
         pb('\033[;;;;4:3;38:5:10;48:2:1:2:3;1$r',
-           ('deccara', '0 0 0 0 4 3 '), ('deccara', '0 0 0 0 38 5 10 '), ('deccara', '0 0 0 0 48 2 1 2 3 '), ('deccara', '0 0 0 0 1 '))
+           ('deccara', '0 0 0 0 4:3'), ('deccara', '0 0 0 0 38:5:10'), ('deccara', '0 0 0 0 48:2:1:2:3'), ('deccara', '0 0 0 0 1'))
         for y in range(s.lines):
             line = s.line(y)
             for x in range(s.columns):
@@ -469,7 +629,7 @@ class TestParser(BaseTest):
                 self.ae(c.fg, (10 << 8) | 1)
                 self.ae(c.bg, (1 << 24 | 2 << 16 | 3 << 8 | 2))
         self.ae(s.line(0).cursor_from(0).bold, True)
-        pb('\033[1;2;2;3;22;39$r', ('deccara', '1 2 2 3 22 '), ('deccara', '1 2 2 3 39 '))
+        pb('\033[1;2;2;3;22;39$r', ('deccara', '1 2 2 3 22 39'))
         self.ae(s.line(0).cursor_from(0).bold, True)
         line = s.line(0)
         for x in range(1, s.columns):
@@ -481,7 +641,7 @@ class TestParser(BaseTest):
             c = line.cursor_from(x)
             self.ae(c.bold, False)
         self.ae(line.cursor_from(3).bold, True)
-        pb('\033[2*x\033[3;2;4;3;34$r\033[*x', ('screen_decsace', 2), ('deccara', '3 2 4 3 34 '), ('screen_decsace', 0))
+        pb('\033[2*x\033[3;2;4;3;34$r\033[*x', ('screen_decsace', 2), ('deccara', '3 2 4 3 34'), ('screen_decsace', 0))
         for y in range(2, 4):
             line = s.line(y)
             for x in range(s.columns):
